# Contributing

This contains useful information for anyone looking to contribute to the project,
or anyone looking for more insight into how the project works.

Please feel free to suggest any changes you would like to see.
As we all have our own use cases for tools, any outside information is high appreciated.

## Terminology

A short list of terms used to within the project that may have ambiguous definitions.
This is only meant to add clarity to the documentation, nothing more.

| Name                  | Definition                                        |
| ---                   | ---                                               |
| __strictly__          | non-optional                                      |
| __likely__            | highly probable                                   |
| __should__            | encouraged on a best effort basis                 |
| __should_not__        | discouraged on a best effort basis                |
| __patch__             | third part of the version number                  |
| __minor__             | second part of the version number                 |
| __major__             | first part of the version number                  |
| __version_bup__       | point at which __patch__ increments               |
| __version_release__   | point at which the version is released            |
| __beta__              | not quiet fully tested but __likely__ to be fine  |
| __stable__            | is tested and maintains `semver` compatibility    |
| __fixed__             | __strictly__ no external behaviour or API changes |

### Clarity

Some further information regarding the table above.

### __version_bup__ vs __version_release__

__Why are these separate?__

There is a short window of time when a release has been published (invoking a __version_bup__),
however the build has yet to make it onto the build server.
If __strictly__ non-breaking patches are found during this window,
we can simply short circuit the release cycle and release those patches (invoking a __version_bup__, again)
before sending to the build server for the final __version_release__.

## Versioning

Semver __should__ be followed, except __major__ version `0` is not treated differently.
This is primarily due to wanting to get off __major__ version `0` as soon as the next stable project spec is released,
which __should__ provide better definitions and __should_not__ change often.
This would result in __major__ version `1` being __stable__ for the foreseeable future.

## Missing Releases

If you are wondering what happened to the versions `0.0.*`.
They were a mess of prototyping implementations ranging from `sed` and `jq` expressions to overly complex PHP scripts.
It's best that they never get used.

Other releases may jump version numbers rather rapidly (e.g. `0.1.5` -> `0.1.8`).
These would __likely__ be due to non-breaking changes being ready post __version_bup__ but pre __version_release__.
Don't be afraid of __patch__ version jumps, as long as the __major__ version stays consistent,
breaking changes __should_not__ be introduced (with the possible exception of `0.1.*` -> `0.2.*`).

## Current Release

The current release actively maintained.

__Version__: `0.1.*`

__Description__:

While `0.1.*` is functional and there __should_not__ be any external breaking changes,
the internals could definitely be cleaned up and condensed a bit more.
This should be viewed as a __beta__ release rather than a proper __stable__ release,
that will come with `0.2.*`.

## To Do

A simple list of tasks that __should__ be addressed before the release version specified.

> This may change depending on the needs at the time.

### 0.2.0

The first __fixed__ __stable__ release.

- [ ] Cleanup `composer2rpm` internal calls
- [ ] Extend accepted options for `composer2rpm` to improve flexibility
- [ ] Improve error handling in `composer2rpm` to give more useful information
- [ ] Improve in-code documentation
- [ ] Move to git based build solution for `*-rc` releases
- [ ] Use different COPR project for __stable__ releases
- [ ] Create test system to build, install and consume composer packages
    - This could also double as scripted examples for new users

### 0.2.*

The refining and compliance releases.
These are probably the most important releases, as they __should__ enable the packages to gain more exposure.
They __should__ also be accompanied by blog posts and clear documentation.

- [ ] Ensure the `fedora-review` checks are all good
- [ ] Propose inclusion in official Fedora repos
